# Copyright (c) 2021-2023 Alibaba Cloud Computing Ltd.
# SPDX-License-Identifier: MulanPSL-2.0
#!/usr/bin/python3
#-*- coding: utf-8 -*-

import re
import sys
import subprocess
import logging


logger = logging.getLogger(__name__)

# const

class Benchmark():
    def __init__(self, url):
        """Init benchmark

        Args:
            url (string): url
        """
        self.url = url

    def prepare(self):
        # clean up mysql database, and Build database tables and indexes 
        cmd = 'cd /root/benchmarksql/run; sh set_up.sh'
        print(cmd)
        logger.info(cmd)
        result = subprocess.run(
                    cmd,
                    shell=True,
                    close_fds=True,
                    stderr=subprocess.PIPE,
                    stdout=subprocess.PIPE
                )
        self.out = result.stdout.decode('UTF-8','strict')
        self.error = result.stderr.decode('UTF-8','strict')
        return True if result.returncode == 0 else False

    def mysql_build(self):
        cmd = 'cd /root/benchmarksql/run; bash runDatabaseBuild.sh mysql.properties'
        print(cmd)
        logger.info(cmd)
        result = subprocess.run(
                    cmd,
                    shell=True,
                    close_fds=True,
                    stderr=subprocess.PIPE,
                    stdout=subprocess.PIPE
                )
        self.out = result.stdout.decode('UTF-8','strict')
        self.error = result.stderr.decode('UTF-8','strict')
        return True if result.returncode == 0 else False

    def run(self):
        """Run benchmark and parse output
        
        Return True and score list if running benchmark successfully, otherwise return False and empty list.
        """
        status1 = self.prepare()
        status2 = self.mysql_build()
        result = status1 and status2
        if not result:
            return False, []
        cmd = 'cd /root/benchmarksql/run; bash runBenchmark.sh mysql.properties'
        logger.info(cmd)
        result = subprocess.run(
                    cmd, 
                    shell=True,
                    close_fds=True,
                    stderr=subprocess.PIPE,
                    stdout=subprocess.PIPE
                )
        self.out = result.stdout.decode('UTF-8','strict')
        self.error = result.stderr.decode('UTF-8','strict')
        if result.returncode == 0:
            logger.info(self.out)
            
            pattern_Measured_tpmC = re.compile(r'Measured tpmC \(NewOrders\) =\s+([\d.]+)')
            pattern_Measured_tpmTOTAL = re.compile(r'Measured tpmTOTAL =\s+([\d.]+)')
            
            if not re.search(pattern_Measured_tpmC,self.out) \
                or not re.search(pattern_Measured_tpmTOTAL,self.out):
                print("parse error")
                logger.error("can not parse output: {}".format(self.out))
                return False, []
            
            Measured_tpmC = float(re.search(pattern_Measured_tpmC,self.out).group(1))
            
            Measured_tpmTOTAL = float(re.search(pattern_Measured_tpmTOTAL,self.out).group(1))
            
            result = {
                "Measured_tpmC": Measured_tpmC,
                "Measured_tpmTOTAL": Measured_tpmTOTAL,
            }
            
            result_str = ", ".join(["{} = {}".format(k,v) for k,v in result.items()])
            print(result_str)
            return True, result_str

        else:
            logger.error(self.error)
            return False, []

if __name__ == "__main__":
    if sys.argv.__len__() <= 1:
        print("'Target ip' is wanted: python3 vm——mysql_tpcc_base.py [Target ip]")
        exit(1)
    bench = Benchmark(sys.argv[1])
    suc, result = bench.run()
